@@ -23,12 +23,13 @@
 #include <interfaces/idocumentcontroller.h>
 #include <interfaces/icore.h>
 #include <editor/modificationrevision.h>
+#include <ktexteditor/movinginterface.h>
 
 namespace KDevelop {
     
 static bool onDiskChangesForbidden = false;
 
-QString CodeRepresentation::rangeText(KTextEditor::Range range) const
+QString CodeRepresentation::rangeText(const KTextEditor::Range& range) const
 {
     Q_ASSERT(range.end().line() < lines());
     
@@ -45,11 +46,48 @@ QString CodeRepresentation::rangeText(KTextEditor::Range range) const
     return rangedText;
 }
 
+static void grepLine(const QString& identifier, const QString& lineText, int lineNumber, QVector<SimpleRange>& ret, bool surroundedByBoundary)
+{
+    if (identifier.isEmpty())
+        return;
+
+    int pos = 0;
+    while(true)
+    {
+        pos = lineText.indexOf(identifier, pos);
+        if(pos == -1)
+            break;
+        int start = pos;
+        pos += identifier.length();
+        int end = pos;
+        
+        if(!surroundedByBoundary || ( (end == lineText.length() || !lineText[end].isLetterOrNumber() || lineText[end] != '_')
+                                        && (start-1 < 0 || !lineText[start-1].isLetterOrNumber() || lineText[start-1] != '_')) )
+        {
+            ret << SimpleRange(lineNumber, start, lineNumber, end);
+        }
+    }
+    
+}
+
 class EditorCodeRepresentation : public DynamicCodeRepresentation {
   public:
   EditorCodeRepresentation(KTextEditor::Document* document) : m_document(document) {
       m_url = IndexedString(m_document->url());
   }
+  
+  virtual QVector< SimpleRange > grep ( const QString& identifier, bool surroundedByBoundary ) const {
+      QVector< SimpleRange > ret;
+
+      if (identifier.isEmpty())
+        return ret;
+
+      for(int line = 0; line < m_document->lines(); ++line)
+        grepLine(identifier, m_document->line(line), line, ret, surroundedByBoundary);
+
+      return ret;
+  }
+  
   QString line(int line) const {
         if(line < 0 || line >= m_document->lines())
             return QString();
@@ -64,7 +102,7 @@ class EditorCodeRepresentation : public DynamicCodeRepresentation {
     return m_document->text();
   }
   
-  bool setText(QString text) {
+  bool setText(const QString& text) {
     bool ret = m_document->setText(text);
     ModificationRevision::clearModificationCache(m_url);
     return ret;
@@ -82,7 +120,8 @@ class EditorCodeRepresentation : public DynamicCodeRepresentation {
       m_document->endEditing();
   }
   
-  bool replace(const KTextEditor::Range& range, QString oldText, QString newText, bool ignoreOldText) {
+  bool replace(const KTextEditor::Range& range, const QString& oldText,
+               const QString& newText, bool ignoreOldText) {
       QString old = m_document->text(range);
       if(oldText != old && !ignoreOldText) {
           return false;
@@ -94,7 +133,7 @@ class EditorCodeRepresentation : public DynamicCodeRepresentation {
       return ret;
   }
   
-  virtual QString rangeText(KTextEditor::Range range) const {
+  virtual QString rangeText(const KTextEditor::Range& range) const {
       return m_document->text(range);
   }
   
@@ -105,7 +144,7 @@ class EditorCodeRepresentation : public DynamicCodeRepresentation {
 
 class FileCodeRepresentation : public CodeRepresentation {
   public:
-    FileCodeRepresentation(IndexedString document) : m_document(document) {
+    FileCodeRepresentation(const IndexedString& document) : m_document(document) {
         QString localFile(document.toUrl().toLocalFile());
   
         QFile file( localFile );
@@ -123,6 +162,18 @@ class FileCodeRepresentation : public CodeRepresentation {
       return lineData.at(line);
     }
     
+    virtual QVector< SimpleRange > grep ( const QString& identifier, bool surroundedByBoundary ) const {
+        QVector< SimpleRange > ret;
+
+        if (identifier.isEmpty())
+            return ret;
+
+        for(int line = 0; line < lineData.count(); ++line)
+            grepLine(identifier, lineData.at(line), line, ret, surroundedByBoundary);
+
+        return ret;
+    }
+    
     virtual int lines() const {
         return lineData.count();
     }
@@ -131,7 +182,7 @@ class FileCodeRepresentation : public CodeRepresentation {
       return data;
     }
     
-    bool setText(QString text) {
+    bool setText(const QString& text) {
       Q_ASSERT(!onDiskChangesForbidden);
       QString localFile(m_document.toUrl().toLocalFile());
 
@@ -163,19 +214,20 @@ class FileCodeRepresentation : public CodeRepresentation {
 
 class ArtificialStringData : public QSharedData {
     public:
-    ArtificialStringData(QString data) {
+    ArtificialStringData(const QString& data) {
         setData(data);
     }
-    void setData(QString data) {
+    void setData(const QString& data) {
         m_data = data;
         m_lineData = m_data.split('\n');
     }
     QString data() const {
         return m_data;
     }
-    QStringList lines() const {
+    const QStringList& lines() const {
         return m_lineData;
     }
+    
     private:
     QString m_data;
     QStringList m_lineData;
@@ -202,7 +254,7 @@ class StringCodeRepresentation : public CodeRepresentation {
         return data->data();
     }
     
-    bool setText(QString text) {
+    bool setText(const QString& text) {
         data->setData(text);
         return true;
     }
@@ -211,6 +263,18 @@ class StringCodeRepresentation : public CodeRepresentation {
         return false;
     }
     
+    virtual QVector< SimpleRange > grep ( const QString& identifier, bool surroundedByBoundary ) const {
+        QVector< SimpleRange > ret;
+
+        if (identifier.isEmpty())
+            return ret;
+
+        for(int line = 0; line < data->lines().count(); ++line)
+            grepLine(identifier, data->lines().at(line), line, ret, surroundedByBoundary);
+
+        return ret;
+    }
+    
   private:
     KSharedPtr<ArtificialStringData> data;
 };
@@ -218,7 +282,7 @@ class StringCodeRepresentation : public CodeRepresentation {
 static QHash<IndexedString, KSharedPtr<ArtificialStringData> > artificialStrings;
 
 //Return the representation for the given URL if it exists, or an empty pointer otherwise
-KSharedPtr<ArtificialStringData> representationForUrl(IndexedString url)
+KSharedPtr<ArtificialStringData> representationForUrl(const IndexedString& url)
 {
     if(artificialStrings.contains(url))
         return artificialStrings[url];
@@ -232,12 +296,12 @@ KSharedPtr<ArtificialStringData> representationForUrl(IndexedString url)
     }
 }
 
-bool artificialCodeRepresentationExists(IndexedString url)
+bool artificialCodeRepresentationExists(const IndexedString& url)
 {
     return !representationForUrl(url).isNull();
 }
 
-CodeRepresentation::Ptr createCodeRepresentation(IndexedString url) {
+CodeRepresentation::Ptr createCodeRepresentation(const IndexedString& url) {
     if(artificialCodeRepresentationExists(url))
         return CodeRepresentation::Ptr(new StringCodeRepresentation(representationForUrl(url)));
 
@@ -253,7 +317,7 @@ void CodeRepresentation::setDiskChangesForbidden(bool changesForbidden)
     onDiskChangesForbidden = changesForbidden;
 }
 
-KUrl CodeRepresentation::artificialUrl(const QString & name)
+KUrl CodeRepresentation::artificialUrl(const QString& name)
 {
     KUrl url(name);
     url.setScheme("artificial");
@@ -262,7 +326,10 @@ KUrl CodeRepresentation::artificialUrl(const QString & name)
     return url;
 }
 
-InsertArtificialCodeRepresentation::InsertArtificialCodeRepresentation(IndexedString file, QString text) : m_file(file) {
+InsertArtificialCodeRepresentation::InsertArtificialCodeRepresentation(const IndexedString& file,
+                                                                       const QString& text)
+: m_file(file)
+{
     if(m_file.toUrl().isRelative())
     {
         m_file = IndexedString(CodeRepresentation::artificialUrl(file.str()));
@@ -290,7 +357,7 @@ InsertArtificialCodeRepresentation::~InsertArtificialCodeRepresentation() {
     artificialStrings.remove(m_file);
 }
 
-void InsertArtificialCodeRepresentation::setText(QString text) {
+void InsertArtificialCodeRepresentation::setText(const QString& text) {
     Q_ASSERT(artificialStrings.contains(m_file));
     artificialStrings[m_file]->setData(text);
 }
