@@ -20,17 +20,20 @@
 
 #include <QDataStream>
 #include <QUuid>
+#include <QApplication>
+#include <QTextStream>
 
-#include <kstandarddirs.h>
-#include <kcomponentdata.h>
-#include <klockfile.h>
-#include <kmessagebox.h>
-#include <klocale.h>
+#include <KStandardDirs>
+#include <KComponentData>
+#include <KLockFile>
+#include <KMessageBox>
+#include <KLocale>
 
 #include <interfaces/icore.h>
+#include <interfaces/isession.h>
+#include <util/fileutils.h>
 
 #include "../duchain.h"
-#include <interfaces/isession.h>
 
 namespace KDevelop {
 
@@ -39,7 +42,7 @@ const int crashesBeforeCleanup = 2;
 
 uint staticItemRepositoryVersion() {
   //Increase this to reset incompatible item-repositories
-  return 67;
+  return 71;
 }
 
 AbstractItemRepository::~AbstractItemRepository() {
@@ -178,42 +181,9 @@ void ItemRepositoryRegistry::unRegisterRepository(AbstractItemRepository* reposi
   m_repositories.remove(repository);
 }
 
-//Recursive delete, copied from a mailing-list
-//Returns true on success
-bool removeDirectory(const QDir &aDir)
-{
-  bool has_err = false;
-  if (aDir.exists())//QDir::NoDotAndDotDot
-  {
-    QFileInfoList entries = aDir.entryInfoList(QDir::NoDotAndDotDot | 
-    QDir::Dirs | QDir::Files);
-    int count = entries.size();
-    for (int idx = 0; ((idx < count) && !has_err); idx++)
-    {
-      QFileInfo entryInfo = entries[idx];
-      QString path = entryInfo.absoluteFilePath();
-      if (entryInfo.isDir())
-      {
-        has_err = !removeDirectory(QDir(path));
-      }
-      else
-      {
-        QFile file(path);
-        if (!file.remove())
-        has_err = true;
-      }
-    }
-    if (!aDir.rmdir(aDir.absolutePath()))
-      has_err = true;
-  }
-  return !has_err;
-}
-
 //After calling this, the data-directory may be a new one
 void ItemRepositoryRegistry::deleteDataDirectory() {
   QMutexLocker lock(&m_mutex);
-  QFileInfo pathInfo(m_path);
-  QDir d(m_path);
 
   //lockForWriting creates a file, that prevents any other KDevelop instance from using the directory as it is.
   //Instead, the other instance will try to delete the directory as well.
@@ -223,7 +193,7 @@ void ItemRepositoryRegistry::deleteDataDirectory() {
   // to be released before deleting a directory that contains these files
   
   m_lock->unlock();
-  bool result = removeDirectory(d);
+  bool result = removeDirectory(m_path);
   Q_ASSERT(result);
   Q_ASSERT(m_lock);
   //Just remove the old directory, and allocate a new one. Probably it'll be the same one.
@@ -232,6 +202,13 @@ void ItemRepositoryRegistry::deleteDataDirectory() {
   m_lock = repo.second;
 }
 
+void setCrashCounter(QFile& crashesFile, int count) {
+  crashesFile.close();
+  crashesFile.open(QIODevice::WriteOnly | QIODevice::Truncate);
+  QDataStream writeStream(&crashesFile);
+  writeStream << count;
+}
+
 bool ItemRepositoryRegistry::open(const QString& path, bool clear, KLockFile::Ptr lock) {
   QMutexLocker mlock(&m_mutex);
   if(m_path == path && !clear)
@@ -276,22 +253,47 @@ bool ItemRepositoryRegistry::open(const QString& path, bool clear, KLockFile::Pt
       
       if(count >= crashesBeforeCleanup && !getenv("DONT_CLEAR_DUCHAIN_DIR"))
       {
-        ///@todo Ask the user
-        kWarning() << "kdevelop crashed" << count << "times in a row with the duchain repository" << m_path << ", clearing it";
-        clear = true;
+        int userAnswer = 0;
+        ///NOTE: we don't want to crash our beloved tools when run in no-gui mode
+        ///NOTE 2: create a better, reusable version of the below for other tools
+        if (QApplication::type() == QApplication::Tty) {
+          // no ui-mode e.g. for duchainify and other tools
+          QTextStream out(stdout);
+          out << i18np("Session crashed %1 time in a row", "Session crashed %1 times in a row", count) << endl;
+          out << endl;
+          QTextStream in(stdin);
+          QString input;
+          while(true) {
+            out << i18n("Clear cache: [Y/n] ") << flush;
+            input = in.readLine().trimmed();
+            if (input.toLower() == "y" || input.isEmpty()) {
+              userAnswer = KMessageBox::Yes;
+              break;
+            } else if (input.toLower() == "n") {
+              userAnswer = KMessageBox::No;
+              break;
+            }
+          }
+        } else {
+          userAnswer = KMessageBox::questionYesNo(0,
+            i18np("The Session crashed once.", "The Session crashed %1 times in a row.", count) + "\n\n" + i18n("The crash may be caused by a corruption of cached data.\n\nPress OK if you want KDevelop to clear the cache, otherwise press Cancel if you are sure the crash has another origin."),
+            i18n("Session crashed"),
+            KStandardGuiItem::ok(),
+            KStandardGuiItem::cancel());
+        }
+        if (userAnswer == KMessageBox::Yes) {
+          clear = true;
+          kDebug() << "User chose to clean repository";
+        } else {
+          setCrashCounter(crashesFile, 1);
+          kDebug() << "User chose to reset crash counter";
+        }
       }else{
-        ///Increase the crash-count. It will be reset of kdevelop is shut down cleanly.
-        ++count;
-        crashesFile.close();
-        crashesFile.open(QIODevice::WriteOnly | QIODevice::Truncate);
-        QDataStream writeStream(&crashesFile);
-        writeStream << count;
+        ///Increase the crash-count. It will be reset if kdevelop is shut down cleanly.
+        setCrashCounter(crashesFile, ++count);
       }
     }else{
-        ///Increase the crash-count. It will be reset of kdevelop is shut down cleanly.
-        crashesFile.open(QIODevice::WriteOnly | QIODevice::Truncate);
-        QDataStream writeStream(&crashesFile);
-        writeStream << 1;
+      setCrashCounter(crashesFile, 1);
     }
     
     if(clear) {
