@@ -58,6 +58,8 @@ ProblemHighlighter::ProblemHighlighter(KTextEditor::Document* document)
             this, SLOT(aboutToInvalidateMovingInterfaceContent()));
     connect(m_document, SIGNAL(aboutToRemoveText(KTextEditor::Range)),
             this, SLOT(aboutToRemoveText(KTextEditor::Range)));
+    connect(m_document, SIGNAL(reloaded(KTextEditor::Document*)),
+            this, SLOT(documentReloaded()));
 }
 
 void ProblemHighlighter::settingsChanged()
@@ -72,7 +74,7 @@ void ProblemHighlighter::viewCreated(Document* , View* view)
     if( !iface )
         return;
 
-    connect(view, SIGNAL(needTextHint(const KTextEditor::Cursor&, QString&)), this, SLOT(textHintRequested(const KTextEditor::Cursor&, QString&)));
+    connect(view, SIGNAL(needTextHint(KTextEditor::Cursor,QString&)), this, SLOT(textHintRequested(KTextEditor::Cursor,QString&)));
 }
 
 void ProblemHighlighter::textHintRequested(const KTextEditor::Cursor& pos, QString& )
@@ -235,4 +237,9 @@ void ProblemHighlighter::aboutToRemoveText( const KTextEditor::Range& range )
     }
 }
 
+void ProblemHighlighter::documentReloaded()
+{
+    setProblems(m_problems);
+}
+
 #include "problemhighlighter.moc"
