@@ -24,11 +24,6 @@
 
 #include <QtGui/QTreeView>
 
-#include <KUrl>
-#include <KTextEditor/Range>
-
-#include <language/interfaces/iproblem.h>
-
 namespace KDevelop {
   class ParseJob;
   class TopDUContext;
@@ -47,28 +42,20 @@ public:
 
   class ProblemModel* model() const;
 
-  static void collectProblems(QList<KDevelop::ProblemPointer>& allProblems, KDevelop::TopDUContext* context, QSet<KDevelop::TopDUContext*>& hadContexts);
-
   virtual void contextMenuEvent(QContextMenuEvent* );
+  virtual void dataChanged(const QModelIndex& topLeft, const QModelIndex& bottomRight);
+  virtual void reset();
   
 protected:
   virtual void showEvent(QShowEvent* event);
 
 private Q_SLOTS:
   void itemActivated(const QModelIndex& index);
-
-  void parseJobFinished(KDevelop::ParseJob* job);
-
-  void documentActivated(KDevelop::IDocument*);
-  
-  void forceFullUpdate();
   
 private:
-  void showProblems(KDevelop::TopDUContext* ctx, KDevelop::IDocument* doc);
+  void resizeColumns();
   ProblemReporterPlugin* m_plugin;
-  KUrl m_activeDirectory;
-  KUrl m_activeUrl;
-  KAction* m_fullUpdateAction;
+;
 };
 
 #endif
