@@ -23,7 +23,7 @@
 
 #include <kcomponentdata.h>
 #include <core.h>
-#include <QtCore/QPointer>
+#include <QWeakPointer>
 
 namespace KDevelop
 {
@@ -47,21 +47,21 @@ class KDEVPLATFORMSHELL_EXPORT CorePrivate {
 public:
     CorePrivate(Core *core);
     ~CorePrivate();
-    bool initialize( Core::Setup mode, const QString& session );
-    QPointer<PluginController> pluginController;
-    QPointer<UiController> uiController;
-    QPointer<ProjectController> projectController;
-    QPointer<LanguageController> languageController;
-    QPointer<PartController> partController;
-    QPointer<DocumentController> documentController;
-    QPointer<RunController> runController;
-    QPointer<SessionController> sessionController;
-    QPointer<SourceFormatterController> sourceFormatterController;
-    QPointer<ProgressManager> progressController;
-    QPointer<SelectionController> selectionController;
-    QPointer<DocumentationController> documentationController;
-    QPointer<DebugController> debugController;
-    QPointer<WorkingSetController> workingSetController;
+    bool initialize( Core::Setup mode, QString session );
+    QWeakPointer<PluginController> pluginController;
+    QWeakPointer<UiController> uiController;
+    QWeakPointer<ProjectController> projectController;
+    QWeakPointer<LanguageController> languageController;
+    QWeakPointer<PartController> partController;
+    QWeakPointer<DocumentController> documentController;
+    QWeakPointer<RunController> runController;
+    QWeakPointer<SessionController> sessionController;
+    QWeakPointer<SourceFormatterController> sourceFormatterController;
+    QWeakPointer<ProgressManager> progressController;
+    QWeakPointer<SelectionController> selectionController;
+    QWeakPointer<DocumentationController> documentationController;
+    QWeakPointer<DebugController> debugController;
+    QWeakPointer<WorkingSetController> workingSetController;
 
     KComponentData m_componentData;
     Core *m_core;
