@@ -44,6 +44,9 @@ Boston, MA 02110-1301, USA.
 #include <sublime/view.h>
 #include <interfaces/iplugincontroller.h>
 #include <interfaces/iprojectcontroller.h>
+#include <interfaces/ibuddydocumentfinder.h>
+#include <interfaces/iproject.h>
+#include <project/projectmodel.h>
 
 #include "core.h"
 #include "mainwindow.h"
@@ -52,8 +55,11 @@ Boston, MA 02110-1301, USA.
 #include "partcontroller.h"
 #include "savedialog.h"
 #include <kmessagebox.h>
+#include <KIO/Job>
+#include "workingsetcontroller.h"
 
 #include <config-kdevplatform.h>
+
 #if HAVE_KOMPARE
     #include "patchdocument.h"
 #endif
@@ -153,14 +159,33 @@ struct DocumentControllerPrivate {
                 }
 
                 documents.insert(document->url(), document);
+
+                if (!controller->isEmptyDocumentUrl(document->url()))
+                {
+                    fileOpenRecent->addUrl(document->url());
+                }
+
                 break;
             }
         }
     }
 
+    KDevelop::IDocument* findBuddyDocument(const KUrl &url, IBuddyDocumentFinder* finder)
+    {
+        QList<KDevelop::IDocument*> allDocs = controller->openDocuments();
+        foreach( KDevelop::IDocument* doc, allDocs ) {
+            if(finder->areBuddies(url, doc->url())) {
+                return doc;
+            }
+        }
+        return 0;
+    }
+
+
     IDocument* openDocumentInternal( const KUrl & inputUrl, const QString& prefName = QString(),
         const KTextEditor::Range& range = KTextEditor::Range::invalid(), const QString& encoding = "",
-        DocumentController::DocumentActivationParams activationParams = 0 )
+        DocumentController::DocumentActivationParams activationParams = 0,
+        IDocument* buddy = 0)
     {
         IDocument* previousActiveDocument = controller->activeDocument();
         KTextEditor::Cursor previousActivePosition;
@@ -227,6 +252,16 @@ struct DocumentControllerPrivate {
                 }
 
                 mimeType = KMimeType::findByUrl( url );
+                
+                if( !url.isLocalFile() && mimeType->isDefault() )
+                {
+                    // fall back to text/plain, for remote files without extension, i.e. COPYING, LICENSE, ...
+                    // using a syncronous KIO::MimetypeJob is hazardous and may lead to repeated calls to
+                    // this function without it having returned in the first place
+                    // and this function is *not* reentrant, see assert below:
+                    // Q_ASSERT(!documents.contains(url) || documents[url]==doc);
+                    mimeType = KMimeType::mimeType("text/plain");
+                }
             }
 
             // is the URL pointing to a directory?
@@ -266,7 +301,7 @@ struct DocumentControllerPrivate {
                     doc = new PartDocument(url, Core::self());
                 } else
                 {
-                    int openAsText = KMessageBox::questionYesNo(0, i18n("KDevelop could not find the editor for file '%1'.\nDo you want to open it as plain text?", url.fileName()), i18n("Could Not Find Editor"));
+                    int openAsText = KMessageBox::questionYesNo(0, i18n("KDevelop could not find the editor for file '%1' of type %2.\nDo you want to open it as plain text?", url.fileName(), mimeType->name()), i18n("Could Not Find Editor"));
                     if (openAsText == KMessageBox::Yes)
                         doc = new TextDocument(url, Core::self(), _encoding);
                     else
@@ -278,7 +313,7 @@ struct DocumentControllerPrivate {
         // The url in the document must equal the current url, else the housekeeping will get broken
         Q_ASSERT(!doc || doc->url() == url);
         
-        if(doc && openDocumentInternal(doc, range, activationParams))
+        if(doc && openDocumentInternal(doc, range, activationParams, buddy))
             return doc;
         else
             return 0;
@@ -287,7 +322,8 @@ struct DocumentControllerPrivate {
     
     bool openDocumentInternal(IDocument* doc,
                                 const KTextEditor::Range& range,
-                                DocumentController::DocumentActivationParams activationParams)
+                                DocumentController::DocumentActivationParams activationParams,
+                                IDocument* buddy = 0)
     {
         IDocument* previousActiveDocument = controller->activeDocument();
         KTextEditor::Cursor previousActivePosition;
@@ -311,7 +347,8 @@ struct DocumentControllerPrivate {
             return false;
         }
         //react on document deletion - we need to cleanup controller structures
-        QObject::connect(sdoc, SIGNAL(aboutToDelete(Sublime::Document*)), controller, SLOT(removeDocument(Sublime::Document*)));
+        
+        QObject::connect(sdoc, SIGNAL(aboutToDelete(Sublime::Document*)), controller, SLOT(notifyDocumentClosed(Sublime::Document*)));
         //We check if it was already opened before
         bool emitOpened = !documents.contains(url);
         if(emitOpened)
@@ -323,7 +360,7 @@ struct DocumentControllerPrivate {
             Sublime::View *partView = 0;
             foreach (Sublime::View *view, sdoc->views())
             {
-                if (area->views().contains(view) && (activationParams.testFlag(DocumentController::DoNotForceCurrentView) || area->indexOf(view) == area->indexOf(uiController->activeSublimeWindow()->activeView())))
+                if (area->views().contains(view) && area->indexOf(view) == area->indexOf(uiController->activeSublimeWindow()->activeView()))
                 {
                     partView = view;
                     break;
@@ -349,15 +386,125 @@ struct DocumentControllerPrivate {
             }
             
             if(addView) {
-                //add view to the area
-                area->addView(partView, uiController->activeSublimeWindow()->activeView());
+                // This code is never executed when restoring session on startup,
+                // only when opening a file manually
+
+                Sublime::View* buddyView = 0;
+                bool placeAfterBuddy = true;
+                if(Core::self()->uiControllerInternal()->arrangeBuddies()) {
+                    // If buddy is not set, look for a (usually) plugin which handles this URL's mimetype
+                    // and use its IBuddyDocumentFinder, if available, to find a buddy document
+                    if(!buddy && doc->mimeType()) {
+                        QString mime = doc->mimeType()->name();
+                        IBuddyDocumentFinder* buddyFinder = IBuddyDocumentFinder::finderForMimeType(mime);
+                        if(buddyFinder) {
+                            buddy = findBuddyDocument(url, buddyFinder);
+                            if(buddy) {
+                                placeAfterBuddy = buddyFinder->buddyOrder(buddy->url(), doc->url());
+                            }
+                        }
+                    }
+
+                    if(buddy) {
+                        Sublime::Document* sublimeDocBuddy = dynamic_cast<Sublime::Document*>(buddy);
+
+                        if(sublimeDocBuddy) {
+                            Sublime::AreaIndex *pActiveViewIndex = area->indexOf(uiController->activeSublimeWindow()->activeView());
+                            if(pActiveViewIndex) {
+                                // try to find existing View of buddy document in current active view's tab
+                                foreach (Sublime::View *pView, pActiveViewIndex->views()) {
+                                    if(sublimeDocBuddy->views().contains(pView)) {
+                                        buddyView = pView;
+                                        break;
+                                    }
+                                }
+                            }
+                            // if we did not find it, then search in the whole area
+                            if(!buddyView) {
+                                foreach (Sublime::View *view, sublimeDocBuddy->views()) {
+                                    if (area->views().contains(view)) {
+                                        buddyView = view;
+                                        break;
+                                    }
+                                }
+                            }
+                        }
+                    }
+                }
+
+                // add view to the area
+                if(buddyView && area->indexOf(buddyView)) {
+                    if(placeAfterBuddy) {
+                        // Adding new view after buddy view, simple case
+                        area->addView(partView, area->indexOf(buddyView), buddyView);
+                    }
+                    else {
+                        // First new view, then buddy view
+                        area->addView(partView, area->indexOf(buddyView), buddyView);
+                        // move buddyView tab after the new document
+                        area->removeView(buddyView);
+                        area->addView(buddyView, area->indexOf(partView), partView);
+                    }
+                }
+                else {
+                    // no buddy found for new document / plugin does not support buddies / buddy feature disabled
+                    Sublime::View *activeView = uiController->activeSublimeWindow()->activeView();
+                    Sublime::UrlDocument *activeDoc = 0;
+                    IBuddyDocumentFinder *buddyFinder = 0;
+                    if(activeView)
+                        activeDoc = dynamic_cast<Sublime::UrlDocument *>(activeView->document());
+                    if(activeDoc && Core::self()->uiControllerInternal()->arrangeBuddies()) {
+                        QString mime = KMimeType::findByUrl(activeDoc->url())->name();
+                        buddyFinder = IBuddyDocumentFinder::finderForMimeType(mime);
+                    }
+
+                    if(Core::self()->uiControllerInternal()->openAfterCurrent() &&
+                       Core::self()->uiControllerInternal()->arrangeBuddies() &&
+                       buddyFinder)
+                    {
+                        // Check if active document's buddy is directly next to it.
+                        // For example, we have the already-open tabs | *foo.h* | foo.cpp | , foo.h is active.
+                        // When we open a new document here (and the buddy feature is enabled),
+                        // we do not want to separate foo.h and foo.cpp, so we take care and avoid this.
+                        Sublime::AreaIndex *activeAreaIndex = area->indexOf(activeView);
+                        int pos = activeAreaIndex->views().indexOf(activeView);
+                        Sublime::View *afterActiveView = activeAreaIndex->views().value(pos+1, 0);
+
+                        Sublime::UrlDocument *activeDoc = 0, *afterActiveDoc = 0;
+                        if(activeView && afterActiveView) {
+                            activeDoc = dynamic_cast<Sublime::UrlDocument *>(activeView->document());
+                            afterActiveDoc = dynamic_cast<Sublime::UrlDocument *>(afterActiveView->document());
+                        }
+                        if(activeDoc && afterActiveDoc &&
+                           buddyFinder->areBuddies(activeDoc->url(), afterActiveDoc->url()))
+                        {
+                            // don't insert in between of two buddies, but after them
+                            area->addView(partView, activeAreaIndex, afterActiveView);
+                        }
+                        else {
+                            // The active document's buddy is not directly after it
+                            // => no ploblem, insert after active document
+                            area->addView(partView, activeView);
+                        }
+                    }
+                    else {
+                        // Opening as last tab won't disturb our buddies
+                        // Same, if buddies are disabled, we needn't care about them.
+
+                        // this method places the tab according to openAfterCurrent()
+                        area->addView(partView, activeView);
+                    }
+                }
             }
             
             if (!activationParams.testFlag(IDocumentController::DoNotActivate))
             {
                 uiController->activeSublimeWindow()->activateView(partView);
             }
-            fileOpenRecent->addUrl( url );
+            if (!controller->isEmptyDocumentUrl(url))
+            {
+                fileOpenRecent->addUrl( url );
+            }
 
             if( applyRange && range.isValid() )
             {
@@ -415,7 +562,6 @@ struct DocumentControllerPrivate {
     void jumpTo( const HistoryEntry & );*/
 };
 
-
 DocumentController::DocumentController( QObject *parent )
         : IDocumentController( parent )
 {
@@ -435,6 +581,7 @@ DocumentController::DocumentController( QObject *parent )
 
 void KDevelop::DocumentController::initialize()
 {
+    connect(ICore::self()->projectController(), SIGNAL(projectOpened(KDevelop::IProject*)), SLOT(slotProjectOpened(KDevelop::IProject*)));
 }
 
 void DocumentController::cleanup()
@@ -465,27 +612,28 @@ void DocumentController::setupActions()
     action->setIcon(KIcon("document-open"));
     action->setShortcut( Qt::CTRL + Qt::Key_O );
     action->setText(i18n( "&Open..." ) );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( chooseDocument() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(chooseDocument()) );
     action->setToolTip( i18n( "Open file" ) );
     action->setWhatsThis( i18n( "<b>Open file</b><p>Opens a file for editing.</p>" ) );
 
     d->fileOpenRecent = KStandardAction::openRecent(this,
-                    SLOT(slotOpenDocument(const KUrl&)), ac);
+                    SLOT(slotOpenDocument(KUrl)), ac);
     d->fileOpenRecent->setWhatsThis(i18n("This lists files which you have opened recently, and allows you to easily open them again."));
     d->fileOpenRecent->loadEntries( KConfigGroup(KGlobal::config(), "Recent Files" ) );
 
     action = d->saveAll = ac->addAction( "file_save_all" );
     action->setIcon(KIcon("document-save"));
     action->setText(i18n( "Save Al&l" ) );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( slotSaveAllDocuments() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(slotSaveAllDocuments()) );
     action->setToolTip( i18n( "Save all open documents" ) );
     action->setWhatsThis( i18n( "<b>Save all documents</b><p>Save all open documents, prompting for additional information when necessary.</p>" ) );
+    action->setShortcut( QKeySequence(Qt::CTRL + Qt::Key_L) );
     action->setEnabled(false);
 
     action = d->revertAll = ac->addAction( "file_revert_all" );
     action->setIcon(KIcon("document-revert"));
-    action->setText(i18n( "Rever&t All" ) );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( reloadAllDocuments() ) );
+    action->setText(i18n( "Reload All" ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(reloadAllDocuments()) );
     action->setToolTip( i18n( "Revert all open documents" ) );
     action->setWhatsThis( i18n( "<b>Revert all documents</b><p>Revert all open documents, returning to the previously saved state.</p>" ) );
     action->setEnabled(false);
@@ -494,7 +642,7 @@ void DocumentController::setupActions()
     action->setIcon(KIcon("window-close"));
     action->setShortcut( Qt::CTRL + Qt::Key_W );
     action->setText( i18n( "&Close" ) );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( fileClose() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(fileClose()) );
     action->setToolTip( i18n( "Close File" ) );
     action->setWhatsThis( i18n( "<b>Close File</b><p>Closes current file.</p>" ) );
     action->setEnabled(false);
@@ -502,7 +650,7 @@ void DocumentController::setupActions()
     action = d->closeAll = ac->addAction( "file_close_all" );
     action->setIcon(KIcon("window-close"));
     action->setText(i18n( "Clos&e All" ) );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( closeAllDocuments() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(closeAllDocuments()) );
     action->setToolTip( i18n( "Close all open documents" ) );
     action->setWhatsThis( i18n( "<b>Close all documents</b><p>Close all open documents, prompting for additional information when necessary.</p>" ) );
     action->setEnabled(false);
@@ -511,7 +659,7 @@ void DocumentController::setupActions()
     action->setIcon(KIcon("window-close"));
     action->setShortcut( Qt::CTRL + Qt::SHIFT + Qt::Key_W );
     action->setText(i18n( "Close All Ot&hers" ) );
-    connect( action, SIGNAL( triggered( bool ) ), SLOT( closeAllOtherDocuments() ) );
+    connect( action, SIGNAL(triggered(bool)), SLOT(closeAllOtherDocuments()) );
     action->setToolTip( i18n( "Close all other documents" ) );
     action->setWhatsThis( i18n( "<b>Close all other documents</b><p>Close all open documents, with the exception of the currently active document.</p>" ) );
     action->setEnabled(false);
@@ -535,10 +683,21 @@ void DocumentController::slotOpenDocument(const KUrl &url)
 IDocument* DocumentController::openDocumentFromText( const QString& data )
 {
     IDocument* d = openDocument(nextEmptyDocumentUrl());
+    Q_ASSERT(d->textDocument());
     d->textDocument()->setText( data );
     return d;
 }
 
+bool DocumentController::openDocumentFromTextSimple( QString text )
+{
+    return (bool)openDocumentFromText( text );
+}
+
+bool DocumentController::openDocumentSimple( QString url )
+{
+    return (bool)openDocument( KUrl(url) );
+}
+
 IDocument* DocumentController::openDocument( const KUrl& inputUrl, const QString& prefName )
 {
     return d->openDocumentInternal( inputUrl, prefName );
@@ -547,17 +706,18 @@ IDocument* DocumentController::openDocument( const KUrl& inputUrl, const QString
 IDocument* DocumentController::openDocument( const KUrl & inputUrl,
         const KTextEditor::Range& range,
         DocumentActivationParams activationParams,
-        const QString& encoding)
+        const QString& encoding, IDocument* buddy)
 {
-    return d->openDocumentInternal( inputUrl, "", range, encoding, activationParams );
+    return d->openDocumentInternal( inputUrl, "", range, encoding, activationParams, buddy);
 }
 
 
 bool DocumentController::openDocument(IDocument* doc,
                                       const KTextEditor::Range& range,
-                                      DocumentActivationParams activationParams)
+                                      DocumentActivationParams activationParams,
+                                      IDocument* buddy)
 {
-    return d->openDocumentInternal( doc, range, activationParams );
+    return d->openDocumentInternal( doc, range, activationParams, buddy);
 }
 
 
@@ -584,10 +744,13 @@ void DocumentController::closeDocument( const KUrl &url )
     d->documents[url]->close();
 }
 
-void DocumentController::notifyDocumentClosed(IDocument* doc)
+void DocumentController::notifyDocumentClosed(Sublime::Document* doc_)
 {
-    d->documents.remove(doc->url());
-
+    IDocument* doc = dynamic_cast<IDocument*>(doc_);
+    Q_ASSERT(doc);
+    
+    d->removeDocument(doc_);
+    
     if (d->documents.isEmpty()) {
         if (d->saveAll)
             d->saveAll->setEnabled(false);
@@ -634,6 +797,7 @@ QList<IDocument*> DocumentController::openDocuments() const
 void DocumentController::activateDocument( IDocument * document, const KTextEditor::Range& range )
 {
     // TODO avoid some code in openDocument?
+    Q_ASSERT(document);
     openDocument(document->url(), range);
 }
 
@@ -692,7 +856,7 @@ QList< IDocument * > KDevelop::DocumentController::documentsInWindow(MainWindow
     return list;
 }
 
-QList< IDocument * > KDevelop::DocumentController::documentsExclusivelyInWindow(MainWindow * mw) const
+QList< IDocument * > KDevelop::DocumentController::documentsExclusivelyInWindow(MainWindow * mw, bool currentAreaOnly) const
 {
     // Gather a list of all documents which have views only in the given main window
     QList<IDocument*> checkSave;
@@ -703,7 +867,7 @@ QList< IDocument * > KDevelop::DocumentController::documentsExclusivelyInWindow(
 
             foreach (Sublime::View* view, sdoc->views()) {
                 foreach(Sublime::MainWindow* window, Core::self()->uiControllerInternal()->mainWindows())
-                    if(window != mw && window->containsView(view))
+                    if(window->containsView(view) && (window != mw || (currentAreaOnly && window == mw && !mw->area()->views().contains(view))))
                         inOtherWindow = true;
             }
 
@@ -723,9 +887,9 @@ QList< IDocument * > KDevelop::DocumentController::modifiedDocuments(const QList
     return ret;
 }
 
-bool DocumentController::saveAllDocumentsForWindow(MainWindow* mw, IDocument::DocumentSaveMode mode)
+bool DocumentController::saveAllDocumentsForWindow(KParts::MainWindow* mw, KDevelop::IDocument::DocumentSaveMode mode, bool currentAreaOnly)
 {
-    QList<IDocument*> checkSave = documentsExclusivelyInWindow(mw);
+    QList<IDocument*> checkSave = documentsExclusivelyInWindow(dynamic_cast<KDevelop::MainWindow*>(mw), currentAreaOnly);
 
     return saveSomeDocuments(checkSave, mode);
 }
@@ -791,6 +955,26 @@ IDocument* DocumentController::activeDocument() const
     return dynamic_cast<IDocument*>(uiController->activeSublimeWindow()->activeView()->document());
 }
 
+QString DocumentController::activeDocumentPath() const
+{
+    IDocument* doc = activeDocument();
+    if(!doc)
+        return QString();
+    return doc->url().pathOrUrl();
+}
+
+QStringList DocumentController::activeDocumentPaths() const
+{
+    UiController *uiController = Core::self()->uiControllerInternal();
+    if( !uiController->activeSublimeWindow() ) return QStringList();
+    
+    QSet<QString> documents;
+    foreach(Sublime::View* view, uiController->activeSublimeWindow()->area()->views())
+        documents.insert(KUrl(view->document()->documentSpecifier()).pathOrUrl());
+    
+    return documents.toList();
+}
+
 void DocumentController::registerDocumentForMimetype( const QString& mimetype,
                                         KDevelop::IDocumentFactory* factory )
 {
@@ -840,10 +1024,181 @@ IDocumentFactory* DocumentController::factory(const QString& mime) const
 KTextEditor::Document* DocumentController::globalTextEditorInstance()
 {
     if(!d->globalTextEditorInstance)
-        d->globalTextEditorInstance = Core::self()->partControllerInternal()->createTextPart(QString());
+        d->globalTextEditorInstance = Core::self()->partControllerInternal()->createTextPart();
     return d->globalTextEditorInstance;
 }
 
+bool DocumentController::openDocumentsSimple( QStringList urls )
+{
+    Sublime::Area* area = Core::self()->uiControllerInternal()->activeArea();
+    Sublime::AreaIndex* areaIndex = area->rootIndex();
+
+    if(Sublime::View* activeView = Core::self()->uiControllerInternal()->activeSublimeWindow()->activeView())
+        areaIndex = area->indexOf(activeView);
+
+    kDebug() << "opening " << urls << " to area " << area << " index " << areaIndex << " with children " << areaIndex->first() << " " << areaIndex->second();
+    
+    bool isFirstView = true;
+    
+    bool ret = openDocumentsWithSplitSeparators( areaIndex, urls, isFirstView );
+    
+    kDebug() << "area arch. after opening: " << areaIndex->print();
+    
+    // Required because sublime sometimes doesn't update correctly when the area-index contents has been changed
+    // (especially when views have been moved to other indices, through unsplit, split, etc.)
+    static_cast<Sublime::MainWindow*>(Core::self()->uiControllerInternal()->activeMainWindow())->reconstructViews();
+    
+    return ret;
+}
+
+bool DocumentController::openDocumentsWithSplitSeparators( Sublime::AreaIndex* index, QStringList urlsWithSeparators, bool& isFirstView )
+{
+    kDebug() << "opening " << urlsWithSeparators << " index " << index << " with children " << index->first() << " " << index->second() << " view-count " << index->viewCount();
+    if(urlsWithSeparators.isEmpty())
+        return true;
+    
+    Sublime::Area* area = Core::self()->uiControllerInternal()->activeArea();
+    
+    QList<int> topLevelSeparators; // Indices of the top-level separators (with groups skipped)
+    QStringList separators = QStringList() << "/" << "-";
+    QList<QStringList> groups;
+    
+    bool ret = true;
+    
+    {
+        int parenDepth = 0;
+        int groupStart = 0;
+        for(int pos = 0; pos < urlsWithSeparators.size(); ++pos)
+        {
+            QString item = urlsWithSeparators[pos];
+            if(separators.contains(item))
+            {
+                if(parenDepth == 0)
+                    topLevelSeparators << pos;
+            }else if(item == "[")
+            {
+                if(parenDepth == 0)
+                    groupStart = pos+1;
+                ++parenDepth;
+            }
+            else if(item == "]")
+            {
+                if(parenDepth > 0)
+                {
+                    --parenDepth;
+                    
+                    if(parenDepth == 0)
+                        groups << urlsWithSeparators.mid(groupStart, pos-groupStart);
+                }
+                else{
+                    kDebug() << "syntax error in " << urlsWithSeparators << ": parens do not match";
+                    ret = false;
+                }
+            }else if(parenDepth == 0)
+            {
+                groups << (QStringList() << item);
+            }
+        }
+    }
+    
+    if(topLevelSeparators.isEmpty())
+    {
+        if(urlsWithSeparators.size() > 1)
+        {
+            foreach(QStringList group, groups)
+                ret &= openDocumentsWithSplitSeparators( index, group, isFirstView );
+        }else{
+            while(index->isSplitted())
+                index = index->first();
+            // Simply open the document into the area index
+            IDocument* doc = Core::self()->documentControllerInternal()->openDocument(KUrl(urlsWithSeparators.front()),
+                        KTextEditor::Cursor::invalid(), IDocumentController::DoNotActivate | IDocumentController::DoNotCreateView);
+            Sublime::Document *sublimeDoc = dynamic_cast<Sublime::Document*>(doc);
+            if (sublimeDoc) {
+                Sublime::View* view = sublimeDoc->createView();
+                area->addView(view, index);
+                if(isFirstView)
+                {
+                    static_cast<Sublime::MainWindow*>(Core::self()->uiControllerInternal()->activeMainWindow())->activateView(view);
+                    isFirstView = false;
+                }
+            }else{
+                ret = false;
+            }
+        }
+        return ret;
+    }
+    
+    // Pick a separator in the middle
+    
+    int pickSeparator = topLevelSeparators[topLevelSeparators.size()/2];
+    
+    bool activeViewToSecondChild = false;
+    if(pickSeparator == urlsWithSeparators.size()-1)
+    {
+        // There is no right child group, so the right side should be filled with the currently active views
+        activeViewToSecondChild = true;
+    }else{
+        QStringList separatorsAndParens = separators;
+        separatorsAndParens << "[" << "]";
+        // Check if the second child-set contains an unterminated separator, which means that the active views should end up there
+        for(int pos = pickSeparator+1; pos < urlsWithSeparators.size(); ++pos)
+            if( separators.contains(urlsWithSeparators[pos]) && (pos == urlsWithSeparators.size()-1 ||
+                separatorsAndParens.contains(urlsWithSeparators[pos-1]) ||
+                separatorsAndParens.contains(urlsWithSeparators[pos-1])) )
+                    activeViewToSecondChild = true;
+    }
+    
+    Qt::Orientation orientation = urlsWithSeparators[pickSeparator] == "/" ? Qt::Horizontal : Qt::Vertical;
+    
+    if(!index->isSplitted())
+    {
+        kDebug() << "splitting " << index << "orientation" << orientation << "to second" << activeViewToSecondChild;
+        index->split(orientation, activeViewToSecondChild);
+    }else{
+        index->setOrientation(orientation);
+        kDebug() << "WARNING: Area is already splitted (shouldn't be)" << urlsWithSeparators;
+    }
+    
+    openDocumentsWithSplitSeparators( index->first(), urlsWithSeparators.mid(0, pickSeparator) , isFirstView );
+    if(pickSeparator != urlsWithSeparators.size() - 1)
+        openDocumentsWithSplitSeparators( index->second(), urlsWithSeparators.mid(pickSeparator+1, urlsWithSeparators.size() - (pickSeparator+1) ), isFirstView );
+    
+    // Clean up the child-indices, because document-loading may fail
+    
+    if(!index->first()->viewCount() && !index->first()->isSplitted())
+    {
+        kDebug() << "unsplitting first";
+        index->unsplit(index->first());
+    }
+    else if(!index->second()->viewCount() && !index->second()->isSplitted())
+    {
+        kDebug() << "unsplitting second";
+        index->unsplit(index->second());
+    }
+
+    return ret;
+}
+
+void DocumentController::slotProjectOpened(IProject* p)
+{
+    connect(p->managerPlugin(), SIGNAL(fileRenamed(KUrl,KDevelop::ProjectFileItem*)), SLOT(slotFileRenamed(KUrl,KDevelop::ProjectFileItem*)));
+}
+
+void DocumentController::slotFileRenamed(const KUrl& oldname, ProjectFileItem* newitem)
+{
+    IDocument* doc=documentForUrl(oldname);
+    if(!doc)
+        return;
+    
+    KTextEditor::Cursor c;
+    KTextEditor::Document* textdoc=doc->textDocument();
+    if(textdoc && textdoc->activeView())
+        c = textdoc->activeView()->cursorPosition();
+    
+    doc->close(IDocument::Default);
+    ICore::self()->documentController()->openDocument(newitem->url(), c);
+}
 
 }
 
