@@ -51,7 +51,6 @@ public:
     virtual bool close(DocumentSaveMode mode = Default);
 
     virtual bool save(DocumentSaveMode mode = Default);
-    virtual void reload();
     virtual DocumentState state() const;
 
     virtual KTextEditor::Cursor cursorPosition() const;
@@ -68,6 +67,9 @@ public:
 
     virtual QString documentType() const;
 
+public Q_SLOTS:
+    virtual void reload();
+
 protected:
     virtual Sublime::View *newView(Sublime::Document *doc);
 
@@ -80,7 +82,7 @@ private:
     Q_PRIVATE_SLOT(d, void slotDocumentLoaded())
     Q_PRIVATE_SLOT(d, void documentSaved(KTextEditor::Document*,bool))
 
-    struct TextDocumentPrivate * const d;
+    class TextDocumentPrivate * const d;
     friend class TextDocumentPrivate;
 };
 
