@@ -30,7 +30,6 @@
 #include "sublimedefs.h"
 
 #include "mainwindow.h"
-#include "ideallayout.h"
 #include <qmenubar.h>
 #include <qstylepainter.h>
 #include <qstyleoption.h>
@@ -51,6 +50,7 @@ class Container;
 class Controller;
 class AreaIndex;
 class IdealMainWidget;
+class IdealController;
 
 class AreaTabButton : public QWidget {
     public:
@@ -87,6 +87,12 @@ class AreaTabBar : public QTabBar {
     }
     
     void addCustomTab(QString text, QIcon icon, bool isCurrent, QString areaId, QWidget* customButtonWidget) {
+        // plain sublime doens't provide customButtonWidget, only kdevelop shell does
+        // TODO: adymo: refactor
+        if (customButtonWidget) {
+            customButtonWidget->setParent(this);
+            customButtonWidget->show();
+        }
         areaIds << areaId;
         buttons << new AreaTabButton(text, icon, 16, this, isCurrent, customButtonWidget);
         addTab(QString());
@@ -154,8 +160,13 @@ public:
 
     /**Reconstructs the mainwindow according to the current area.*/
     void reconstruct();
+    /**Reconstructs the views according to the current area index.*/
+    void reconstructViews();    
     /**Clears the area leaving mainwindow empty.*/
     void clearArea();
+    
+    /** Sets a @p w widget that will be shown when there are no documents on the area */
+    void setBackgroundCentralWidget(QWidget* w);
 
     void activateFirstVisibleView();
 
@@ -169,9 +180,11 @@ public:
     View *activeToolView;
 
     QWidget *centralWidget;
+    QWidget* bgCentralWidget;
+    QSplitter* splitterCentralWidget;
     AreaTabWidget* areaSwitcher;
 
-    IdealMainWidget *idealMainWidget;
+    IdealController *idealController;
     int ignoreDockShown;
     bool autoAreaSettingsSave;                       
 
@@ -191,35 +204,30 @@ private slots:
     void switchToArea(QAction *action);
     void updateAreaSwitcher(Sublime::Area *area);
     void slotDockShown(Sublime::View*, Sublime::Position, bool);
-    void widgetResized(IdealMainLayout::Role role, int thickness);
+    void widgetResized(Qt::DockWidgetArea dockArea, int thickness);
     void widgetCloseRequest(QWidget* widget);
 
     void showLeftDock(bool b);
     void showRightDock(bool b);
     void showBottomDock(bool b);
-    void showTopDock(bool b);
-    void anchorCurrentDock(bool b);
-    void maximizeCurrentDock(bool b);
     void focusEditor();
     void toggleDocksShown();
     void selectNextDock();
     void selectPreviousDock();
-    void removeView();
-
-protected:
-    virtual bool eventFilter(QObject *, QEvent *event);
 
 private:
+    void setBackgroundVisible(bool v);
     Qt::DockWidgetArea positionToDockArea(Position position);
-    void recreateCentralWidget();
+    void cleanCentralWidget();
 
     MainWindow *m_mainWindow;
-    QMap<AreaIndex*, QSplitter*> m_indexSplitters;
+    // uses QPointer to make already-deleted splitters detectable
+    QMap<AreaIndex*, QPointer<QSplitter> > m_indexSplitters;
     friend class AreaSelectionAction;
 
     QMap<Area*, QAction*> m_areaActions;
     QMap<QAction*, Area*> m_actionAreas;
-    QPointer<QWidget> m_leftTabbarCornerWidget;
+    QWeakPointer<QWidget> m_leftTabbarCornerWidget;
 };
 
 }
